#!/usr/bin/perl -w

# message.pl - an client for LCDproc
#              version 0.0.3
#
# This client for LCDproc displays a file as a message.
#
#
# Copyright (c) 1999, William Ferrell, Scott Scriven
#               2001, David Glaude
#               2001, Jarda Benkovsky
#               2002, Jonathan Oxer
#               2002, Rene Wagner <reenoo@gmx.de>
#               2005, Sebastian Foelz <foelz@yahoo.com>
#
# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

use IO::Socket;
use Fcntl;

############################################################
# Configurable part. Set it according your setup.
############################################################

# Host which runs lcdproc daemon (LCDd)
$HOST = "localhost";

# Port on which LCDd listens to requests
$PORT = "13666";

# These define the visible part of the file...
$top = 3;    # How far from the end of the file should we start?
$lines = 3;  # How many lines to display?
$left = 1;   # Left/right scrolling position,
$width = 27; # and number of characters per line to show

############################################################
# End of user configurable parts
############################################################
$anz = $#ARGV;
if($anz < 0)
{
    print "Usage: message.pl {options} file\n";
    print "Options: -s <server>\n";
    print "         -p <port>\n";
    print "default is -s localhost -p 13666\n";
    exit;
}

for ($i=0; $i<=$anz; $i++)
{
    $f2 = shift @ARGV;
    if ("$f2" eq "-s")
    {
	$HOST = shift @ARGV;
	$i++;
    }
    if ("$f2" eq "-p")
    {
	$PORT = shift @ARGV;
	$i++;
    }
    $filename = $f2;
}
#$filename = shift @ARGV;

# Connect to the server...
$remote = IO::Socket::INET->new(
		Proto     => "tcp",
		PeerAddr  => $HOST,
		PeerPort  => $PORT,
	)
	|| die "Cannot connect to LCDproc port\n";

# Make sure our messages get there right away
$remote->autoflush(1);

print $remote "hello\n";
# Note: it's good practice to listen for a response after a print to the
# server even if there isn't meant to be one. If you don't, you may find
# your program crashes after running for a while when the buffers fill up.
my $lcdresponse = <$remote>;
#print "LCD   :$lcdresponse.\n";
#print "Remote:$remote.\n";
#print "File  :$filename.\n";


# Turn off blocking mode...
fcntl($remote, F_SETFL, O_NONBLOCK);


# Set up some screen widgets...
print $remote "client_set name {Tail}\n";
$lcdresponse = <$remote>;
print $remote "screen_add tail\n";
$lcdresponse = <$remote>;
print $remote "screen_set tail -priority 16 name {Tail $filename} -heartbeat off\n";
$lcdresponse = <$remote>;
print $remote "widget_add tail title title\n";
$lcdresponse = <$remote>;
print $remote "widget_set tail title {Message:}\n";
$lcdresponse = <$remote>;
print $remote "widget_add tail 1 string\n";
$lcdresponse = <$remote>;
print $remote "widget_add tail 2 string\n";
$lcdresponse = <$remote>;
print $remote "widget_add tail 3 string\n";
$lcdresponse = <$remote>;


# Forever, we should do stuff...
$icd = 10;
while($icd)
{
	# Now, show what the file contains, if anything...
	if( -f $filename )
	{
	    # Grab some text.
	    $right = $left + $width - 1;
	    $text = `tail -$top $filename | head -$lines | cut -c$left-$right`;
	    @lines = split(/\n/, $text);
	    # Now, display that text...
	    for($i=0; $i<$lines; $i++)
	    {
		$j = $i + 1;
		$k = $i + 2;
		if($#lines < $i) { $line = " "; }  # Avoid blank lines
		else { $line = $lines[$i]; }
		if(!($line =~ /\S/)) { $line = " ";}
		print $remote "widget_set tail $j 1 $k {$line }\n";
		my $lcdresponse = <$remote>;
	    }
	}
	else  # If the file is unreadable, show an error
	{
	    print $remote "widget_set tail 1 1 2 {$filename}\n";
		$lcdresponse = <$remote>;
	    print $remote "widget_set tail 2 1 3 {doesn't exist.}\n";
		$lcdresponse = <$remote>;
	    print $remote "widget_set tail 3 1 4 { }\n";
		$lcdresponse = <$remote>;
	}
	sleep 1;
	$icd = $icd - 1;
}

close ($remote)            || die "close: $!";
exit;
